@@ -11,7 +11,6 @@ require 5.000;
 use strict;
 use CGI::Carp qw(fatalsToBrowser warningsToBrowser);
 use CGI::Bus::Base;
-use Net::SMTP;
 use vars qw(@ISA);
 @ISA =qw(CGI::Bus::Base);
 
@@ -25,7 +24,7 @@ use vars qw(@ISA);
 sub smtp {
  my $s =shift;
  $s->set(@_);
- $s->{-smtp} =eval {local $^W=undef; Net::SMTP->new($s->{-host})};
+ $s->{-smtp} =eval {local $^W=undef; eval("use Net::SMTP"); Net::SMTP->new($s->{-host})};
  die("SMTP host '" .$s->{-host} ."' $@\n") if !$s->{-smtp} ||$@;
  $s->{-smtp}
 }
@@ -45,12 +44,24 @@ sub mailsend { # from, to, msg rows
  $s->parent->pushmsg("SMTP msgsend $host $from -> ".join(',',@$to));
  local $^W=undef;
  my $smtp =$s->smtp(); $s->{-smtp} =undef;
- $smtp->mail(index($from,'@') <0 && $dom ? $from .'@' .$dom :$from)
+ # $s->parent->pushmsg("SMTP mail: " .$s->addrtr($from));
+ $smtp->mail($s->addrtr($from))
                             || $s->die("SMTP From: $from\n");
- $smtp->to(map {index($_,'@') <0 && $dom ? $_ .'@' .$dom :$_} @$to)
+ # $s->parent->pushmsg("SMTP to: " .join(',',map {$s->addrtr($_)} @$to));
+ $smtp->to(map {$s->addrtr($_)} @$to)
                             || $s->die("SMTP To: " .join(', ',@$to) ."\n");
  $smtp->data(join("\n",@_)) || $s->die("SMTP Data\n");
  $smtp->dataend()           || $s->die("SMTP DataEnd\n");
  $smtp->quit;
  1
 }
+
+
+sub addrtr {	# address translation
+  ($_[1] =~/^([^\\]+)\\(.+)$/ 
+	? $2 
+	: $_[1])
+ .((index($_[1],'@') <0) && $_[0]->{-domain}
+	? '@' .$_[0]->{-domain} 
+	: '')
+}
\ No newline at end of file
